@@ -50,6 +50,8 @@ private slots:
     void pushSuspendResume_data(){generate_audiofile_testrows();}
     void pushSuspendResume();
 
+    void pushResetResume();
+
     void pushUnderrun_data(){generate_audiofile_testrows();}
     void pushUnderrun();
 
@@ -59,8 +61,16 @@ private slots:
 private:
     using FilePtr = QSharedPointer<QFile>;
 
-    QString formatToFileName(const QAudioFormat &format);
+    static QString formatToFileName(const QAudioFormat &format);
     void createSineWaveData(const QAudioFormat &format, qint64 length, int sampleRate = 440);
+    static QString dumpStateSignalSpy(const QSignalSpy &stateSignalSpy);
+
+    static qint64 wavDataSize(QIODevice &input);
+
+    template<typename Checker>
+    static void pushDataToAudioSink(QAudioSink &sink, QIODevice &input, QIODevice &feed,
+                                    qint64 &allWritten, qint64 writtenLimit, Checker &&checker,
+                                    bool checkOnlyFirst = false);
 
     void generate_audiofile_testrows();
 
@@ -130,15 +140,69 @@ void tst_QAudioSink::createSineWaveData(const QAudioFormat &format, qint64 lengt
     Q_ASSERT(m_buffer->open(QIODevice::ReadOnly));
 }
 
+QString tst_QAudioSink::dumpStateSignalSpy(const QSignalSpy& stateSignalSpy) {
+    QString result = "[";
+    bool first = true;
+    for (auto& params : stateSignalSpy)
+    {
+        if (!std::exchange(first, false))
+            result += ',';
+        result += QString::number(params.front().value<QAudio::State>());
+    }
+    result.append(']');
+    return result;
+}
+
+qint64 tst_QAudioSink::wavDataSize(QIODevice &input)
+{
+    return input.size() - QWaveDecoder::headerLength();
+}
+
+template<typename Checker>
+void tst_QAudioSink::pushDataToAudioSink(QAudioSink &sink, QIODevice &input, QIODevice &feed,
+                                         qint64 &allWritten, qint64 writtenLimit, Checker &&checker,
+                                         bool checkOnlyFirst)
+{
+    bool firstBuffer = true;
+    qint64 offset = 0;
+    QByteArray buffer;
+
+    while ((allWritten < writtenLimit || writtenLimit < 0) && !input.atEnd()
+           && !QTest::currentTestFailed()) {
+        if (sink.bytesFree() > 0) {
+            if (buffer.isNull())
+                buffer = input.read(sink.bytesFree());
+
+            const auto written = feed.write(buffer);
+            allWritten += written;
+            offset += written;
+
+            if (offset >= buffer.size()) {
+                offset = 0;
+                buffer.clear();
+            }
+
+            if (!checkOnlyFirst || firstBuffer)
+                checker();
+
+            firstBuffer = false;
+        } else {
+            // wait a bit to ensure some the sink has consumed some data
+            // The delay getting might need some improvements
+            const auto delay = qMin(10, sink.format().durationForBytes(sink.bufferSize()) / 1000 / 2);
+            QTest::qWait(delay);
+        }
+    }
+}
+
 void tst_QAudioSink::generate_audiofile_testrows()
 {
     QTest::addColumn<FilePtr>("audioFile");
     QTest::addColumn<QAudioFormat>("audioFormat");
 
-    for (int i=0; i<audioFiles.count(); i++) {
+    for (int i=0; i<audioFiles.size(); i++) {
         QTest::newRow(QString("Audio File %1").arg(i).toUtf8().constData())
                 << audioFiles.at(i) << testFormats.at(i);
-
     }
 }
 
@@ -199,7 +263,7 @@ void tst_QAudioSink::initTestCase()
     QVERIFY(m_temporaryDir->isValid());
 
     const QString temporaryAudioPath = m_temporaryDir->path() + slash;
-    for (const QAudioFormat &format : qAsConst(testFormats)) {
+    for (const QAudioFormat &format : std::as_const(testFormats)) {
         qint64 len = format.sampleRate()*format.bytesPerFrame(); // 1 second
         createSineWaveData(format, len);
         // Write generate sine wave data to file
@@ -313,7 +377,7 @@ void tst_QAudioSink::stopWhileStopped()
     audioOutput.stop();
 
     // Check that no state transition occurred
-    QVERIFY2((stateSignal.count() == 0), "stop() while stopped is emitting a signal and it shouldn't");
+    QVERIFY2((stateSignal.size() == 0), "stop() while stopped is emitting a signal and it shouldn't");
     QVERIFY2((audioOutput.error() == QAudio::NoError), "error() was not set to QAudio::NoError after stop()");
 }
 
@@ -333,7 +397,7 @@ void tst_QAudioSink::suspendWhileStopped()
     audioOutput.suspend();
 
     // Check that no state transition occurred
-    QVERIFY2((stateSignal.count() == 0), "stop() while suspended is emitting a signal and it shouldn't");
+    QVERIFY2((stateSignal.size() == 0), "stop() while suspended is emitting a signal and it shouldn't");
     QVERIFY2((audioOutput.error() == QAudio::NoError), "error() was not set to QAudio::NoError after stop()");
 }
 
@@ -353,7 +417,7 @@ void tst_QAudioSink::resumeWhileStopped()
     audioOutput.resume();
 
     // Check that no state transition occurred
-    QVERIFY2((stateSignal.count() == 0), "resume() while stopped is emitting a signal and it shouldn't");
+    QVERIFY2((stateSignal.size() == 0), "resume() while stopped is emitting a signal and it shouldn't");
     QVERIFY2((audioOutput.error() == QAudio::NoError), "error() was not set to QAudio::NoError after resume()");
 }
 
@@ -380,8 +444,11 @@ void tst_QAudioSink::pull()
     audioOutput.start(audioFile.data());
 
     // Check that QAudioSink immediately transitions to ActiveState
-    QTRY_VERIFY2((stateSignal.count() == 1),
-                 QString("didn't emit signal on start(), got %1 signals instead").arg(stateSignal.count()).toUtf8().constData());
+    QTRY_VERIFY2((stateSignal.size() == 1),
+                 QString("didn't emit signal on start(), got %1 signals instead")
+                         .arg(dumpStateSignalSpy(stateSignal))
+                         .toUtf8()
+                         .constData());
     QVERIFY2((audioOutput.state() == QAudio::ActiveState), "didn't transition to ActiveState after start()");
     QVERIFY2((audioOutput.error() == QAudio::NoError), "error state is not equal to QAudio::NoError after start()");
     stateSignal.clear();
@@ -392,8 +459,8 @@ void tst_QAudioSink::pull()
 
     // Wait until playback finishes
     QTRY_VERIFY2(audioFile->atEnd(), "didn't play to EOF");
-    QTRY_VERIFY(stateSignal.count() > 0);
-    QCOMPARE(qvariant_cast<QAudio::State>(stateSignal.last().at(0)), QAudio::IdleState);
+    QTRY_VERIFY(stateSignal.size() > 0);
+    QTRY_COMPARE(qvariant_cast<QAudio::State>(stateSignal.last().at(0)), QAudio::IdleState);
     QVERIFY2((audioOutput.state() == QAudio::IdleState), "didn't transitions to IdleState when at EOF");
     stateSignal.clear();
 
@@ -401,8 +468,11 @@ void tst_QAudioSink::pull()
 
     audioOutput.stop();
     QTest::qWait(40);
-    QVERIFY2((stateSignal.count() == 1),
-             QString("didn't emit StoppedState signal after stop(), got %1 signals instead").arg(stateSignal.count()).toUtf8().constData());
+    QVERIFY2((stateSignal.size() == 1),
+             QString("didn't emit StoppedState signal after stop(), got %1 signals instead")
+                     .arg(dumpStateSignalSpy(stateSignal))
+                     .toUtf8()
+                     .constData());
     QVERIFY2((audioOutput.state() == QAudio::StoppedState), "didn't transitions to StoppedState after stop()");
 
     QVERIFY2((audioOutput.error() == QAudio::NoError), "error() is not QAudio::NoError after stop()");
@@ -432,21 +502,25 @@ void tst_QAudioSink::pullSuspendResume()
 
     audioOutput.start(audioFile.data());
     // Check that QAudioSink immediately transitions to ActiveState
-    QTRY_VERIFY2((stateSignal.count() == 1),
-                 QString("didn't emit signal on start(), got %1 signals instead").arg(stateSignal.count()).toUtf8().constData());
+    QTRY_VERIFY2((stateSignal.size() == 1),
+                 QString("didn't emit signal on start(), got %1 signals instead")
+                         .arg(dumpStateSignalSpy(stateSignal))
+                         .toUtf8()
+                         .constData());
     QVERIFY2((audioOutput.state() == QAudio::ActiveState), "didn't transition to ActiveState after start()");
-    QVERIFY2((audioOutput.error() == QAudio::NoError), "error state is not equal to QAudio::NoError after start()");
-    stateSignal.clear();
+    QVERIFY2((audioOutput.error() == QAudio::NoError),
+             "error state is not equal to QAudio::NoError after start()");
 
+    stateSignal.clear();
     // Wait for half of clip to play
     QTest::qWait(500);
 
     audioOutput.suspend();
     QTest::qWait(100);
 
-    QTRY_VERIFY2((stateSignal.count() == 1),
+    QTRY_VERIFY2((stateSignal.size() == 1),
              QString("didn't emit SuspendedState signal after suspend(), got %1 signals instead")
-             .arg(stateSignal.count()).toUtf8().constData());
+             .arg(dumpStateSignalSpy(stateSignal)).toUtf8().constData());
     QVERIFY2((audioOutput.state() == QAudio::SuspendedState), "didn't transition to SuspendedState after suspend()");
     QVERIFY2((audioOutput.error() == QAudio::NoError), "error state is not equal to QAudio::NoError after suspend()");
     stateSignal.clear();
@@ -461,16 +535,19 @@ void tst_QAudioSink::pullSuspendResume()
     audioOutput.resume();
 
     // Check that QAudioSink immediately transitions to ActiveState
-    QVERIFY2((stateSignal.count() == 1),
-             QString("didn't emit signal after resume(), got %1 signals instead").arg(stateSignal.count()).toUtf8().constData());
+    QVERIFY2((stateSignal.size() == 1),
+             QString("didn't emit signal after resume(), got %1 signals instead")
+                     .arg(dumpStateSignalSpy(stateSignal))
+                     .toUtf8()
+                     .constData());
     QVERIFY2((audioOutput.state() == QAudio::ActiveState), "didn't transition to ActiveState after resume()");
     QVERIFY2((audioOutput.error() == QAudio::NoError), "error state is not equal to QAudio::NoError after resume()");
     stateSignal.clear();
 
     // Wait until playback finishes
     QTRY_VERIFY2(audioFile->atEnd(), "didn't play to EOF");
-    QTRY_VERIFY(stateSignal.count() > 0);
-    QCOMPARE(qvariant_cast<QAudio::State>(stateSignal.last().at(0)), QAudio::IdleState);
+    QTRY_VERIFY(stateSignal.size() > 0);
+    QTRY_COMPARE(qvariant_cast<QAudio::State>(stateSignal.last().at(0)), QAudio::IdleState);
     QVERIFY2((audioOutput.state() == QAudio::IdleState), "didn't transitions to IdleState when at EOF");
     stateSignal.clear();
 
@@ -478,8 +555,11 @@ void tst_QAudioSink::pullSuspendResume()
 
     audioOutput.stop();
     QTest::qWait(40);
-    QVERIFY2((stateSignal.count() == 1),
-             QString("didn't emit StoppedState signal after stop(), got %1 signals instead").arg(stateSignal.count()).toUtf8().constData());
+    QVERIFY2((stateSignal.size() == 1),
+             QString("didn't emit StoppedState signal after stop(), got %1 signals instead")
+                     .arg(dumpStateSignalSpy(stateSignal))
+                     .toUtf8()
+                     .constData());
     QVERIFY2((audioOutput.state() == QAudio::StoppedState), "didn't transitions to StoppedState after stop()");
 
     QVERIFY2((audioOutput.error() == QAudio::NoError), "error() is not QAudio::NoError after stop()");
@@ -488,68 +568,71 @@ void tst_QAudioSink::pullSuspendResume()
     audioFile->close();
 }
 
-class AudioPullSource : public QIODevice
-{
-    Q_OBJECT
-public:
-    qint64 readData(char *data, qint64 len) override {
-        qint64 read = qMin(len, available);
-        available -= read;
-        memset(data, 0, read);
-        return read;
-    }
-    qint64 writeData(const char *, qint64) override { return 0; }
-    bool isSequential() const override { return true; }
-
-    qint64 bytesAvailable() const override { return available; }
-    bool atEnd() const override { return signalEnd && available == 0; }
-
-    qint64 available = 0;
-    bool signalEnd = false;
-};
-
 void tst_QAudioSink::pullResumeFromUnderrun()
 {
-    AudioPullSource audioSource;
+    class AudioPullSource : public QIODevice
+    {
+    public:
+        qint64 readData(char *data, qint64 len) override {
+            qint64 read = qMin(len, available);
+            available -= read;
+            memset(data, 0, read);
+            return read;
+        }
+        qint64 writeData(const char *, qint64) override { return 0; }
+        bool isSequential() const override { return true; }
+
+        qint64 bytesAvailable() const override { return available; }
+        bool atEnd() const override { return signalEnd && available == 0; }
+
+        qint64 available = 0;
+        bool signalEnd = false;
+    };
+
+    constexpr int chunkSize = 128;
+
     QAudioFormat format;
     format.setChannelCount(1);
     format.setSampleFormat(QAudioFormat::UInt8);
-    format.setSampleRate(1024);
-    QAudioSink audioOutput(format, this);
+    format.setSampleRate(8000);
 
+    QAudioSink audioOutput(format, this);
     QSignalSpy stateSignal(&audioOutput, SIGNAL(stateChanged(QAudio::State)));
 
+    AudioPullSource audioSource;
     audioSource.open(QIODeviceBase::ReadOnly);
-    audioSource.available = 128;
+    audioSource.available = chunkSize;
+    QCOMPARE(audioOutput.state(), QAudio::StoppedState);
     audioOutput.start(&audioSource);
 
-    QTRY_VERIFY(stateSignal.count() == 1);
-    QVERIFY(audioOutput.state() == QAudio::ActiveState);
-    QVERIFY(audioOutput.error() == QAudio::NoError);
+    QTRY_COMPARE(stateSignal.size(), 1);
+    QCOMPARE(audioOutput.state(), QAudio::ActiveState);
+    QCOMPARE(audioOutput.error(), QAudio::NoError);
     stateSignal.clear();
 
-    QTRY_VERIFY(stateSignal.count() == 1);
-    QVERIFY(audioOutput.state() == QAudio::IdleState);
-    QVERIFY(audioOutput.error() == QAudio::UnderrunError);
+    QTRY_COMPARE(stateSignal.size(), 1);
+    QCOMPARE(audioOutput.state(), QAudio::IdleState);
+    QCOMPARE(audioOutput.error(), QAudio::UnderrunError);
     stateSignal.clear();
 
     QTest::qWait(300);
-    audioSource.available = 128;
+    audioSource.available = chunkSize;
     audioSource.signalEnd = true;
 
     // Resume pull
     emit audioSource.readyRead();
 
-    QTRY_VERIFY(stateSignal.count() == 1);
-    QVERIFY(audioOutput.state() == QAudio::ActiveState);
-    QVERIFY(audioOutput.error() == QAudio::NoError);
-    stateSignal.clear();
+    QTRY_COMPARE(stateSignal.size(), 2);
+    QCOMPARE(stateSignal.at(0).front().value<QAudio::State>(), QAudio::ActiveState);
+    QCOMPARE(stateSignal.at(1).front().value<QAudio::State>(), QAudio::IdleState);
 
-    QTRY_VERIFY(stateSignal.count() == 1);
-    QVERIFY(audioOutput.state() == QAudio::IdleState);
-    QVERIFY(audioOutput.error() == QAudio::NoError);
+    QCOMPARE(audioOutput.error(), QAudio::NoError);
+    QCOMPARE(audioOutput.state(), QAudio::IdleState);
 
-    QTRY_COMPARE(audioOutput.processedUSecs(), 250000);
+    // we played two chunks, sample rate is per second
+    const int expectedUSecs = (double(chunkSize) / double(format.sampleRate()))
+                            * 2 * 1000 * 1000;
+    QTRY_COMPARE(audioOutput.processedUSecs(), expectedUSecs);
 }
 
 void tst_QAudioSink::push()
@@ -575,8 +658,11 @@ void tst_QAudioSink::push()
     QIODevice* feed = audioOutput.start();
 
     // Check that QAudioSink immediately transitions to IdleState
-    QTRY_VERIFY2((stateSignal.count() == 1),
-                 QString("didn't emit signal on start(), got %1 signals instead").arg(stateSignal.count()).toUtf8().constData());
+    QTRY_VERIFY2((stateSignal.size() == 1),
+                 QString("didn't emit signal on start(), got %1 signals instead")
+                         .arg(dumpStateSignalSpy(stateSignal))
+                         .toUtf8()
+                         .constData());
     QVERIFY2((audioOutput.state() == QAudio::IdleState), "didn't transition to IdleState after start()");
     QVERIFY2((audioOutput.error() == QAudio::NoError), "error state is not equal to QAudio::NoError after start()");
     stateSignal.clear();
@@ -587,33 +673,29 @@ void tst_QAudioSink::push()
     QVERIFY2((audioOutput.processedUSecs() == qint64(0)), "processedUSecs() is not zero after start()");
 
     qint64 written = 0;
-    bool firstBuffer = true;
 
-    while (written < audioFile->size() - QWaveDecoder::headerLength()) {
+    auto checker = [&]() {
+        // Check for transition to ActiveState when data is provided
+        QVERIFY2((stateSignal.size() == 1),
+                 QString("didn't emit signal after receiving data, got %1 signals instead")
+                         .arg(dumpStateSignalSpy(stateSignal))
+                         .toUtf8()
+                         .constData());
+        QVERIFY2((audioOutput.state() == QAudio::ActiveState),
+                 "didn't transition to ActiveState after receiving data");
+        QVERIFY2((audioOutput.error() == QAudio::NoError),
+                 "error state is not equal to QAudio::NoError after receiving data");
+        stateSignal.clear();
+    };
 
-        if (audioOutput.bytesFree() > 0) {
-            auto buffer = audioFile->read(audioOutput.bytesFree());
-            written += feed->write(buffer);
-
-            if (firstBuffer) {
-                // Check for transition to ActiveState when data is provided
-                QVERIFY2((stateSignal.count() == 1),
-                         QString("didn't emit signal after receiving data, got %1 signals instead")
-                         .arg(stateSignal.count()).toUtf8().constData());
-                QVERIFY2((audioOutput.state() == QAudio::ActiveState), "didn't transition to ActiveState after receiving data");
-                QVERIFY2((audioOutput.error() == QAudio::NoError), "error state is not equal to QAudio::NoError after receiving data");
-                firstBuffer = false;
-                stateSignal.clear();
-            }
-        } else
-            QTest::qWait(20);
-    }
+    pushDataToAudioSink(audioOutput, *audioFile, *feed, written, wavDataSize(*audioFile), checker,
+                        true);
 
     // Wait until playback finishes
     QVERIFY2(audioFile->atEnd(), "didn't play to EOF");
     QTRY_VERIFY(audioOutput.state() == QAudio::IdleState);
-    QTRY_VERIFY(stateSignal.count() > 0);
-    QCOMPARE(qvariant_cast<QAudio::State>(stateSignal.last().at(0)), QAudio::IdleState);
+    QTRY_VERIFY(stateSignal.size() > 0);
+    QTRY_COMPARE(qvariant_cast<QAudio::State>(stateSignal.last().at(0)), QAudio::IdleState);
     QVERIFY2((audioOutput.state() == QAudio::IdleState), "didn't transitions to IdleState when at EOF");
     stateSignal.clear();
 
@@ -621,8 +703,8 @@ void tst_QAudioSink::push()
 
     audioOutput.stop();
     QTest::qWait(40);
-    QVERIFY2((stateSignal.count() == 1),
-             QString("didn't emit StoppedState signal after stop(), got %1 signals instead").arg(stateSignal.count()).toUtf8().constData());
+    QVERIFY2((stateSignal.size() == 1),
+             QString("didn't emit StoppedState signal after stop(), got %1 signals instead").arg(dumpStateSignalSpy(stateSignal)).toUtf8().constData());
     QVERIFY2((audioOutput.state() == QAudio::StoppedState), "didn't transitions to StoppedState after stop()");
 
     QVERIFY2((audioOutput.error() == QAudio::NoError), "error() is not QAudio::NoError after stop()");
@@ -654,8 +736,11 @@ void tst_QAudioSink::pushSuspendResume()
     QIODevice* feed = audioOutput.start();
 
     // Check that QAudioSink immediately transitions to IdleState
-    QTRY_VERIFY2((stateSignal.count() == 1),
-                 QString("didn't emit signal on start(), got %1 signals instead").arg(stateSignal.count()).toUtf8().constData());
+    QTRY_VERIFY2((stateSignal.size() == 1),
+                 QString("didn't emit signal on start(), got %1 signals instead")
+                         .arg(dumpStateSignalSpy(stateSignal))
+                         .toUtf8()
+                         .constData());
     QVERIFY2((audioOutput.state() == QAudio::IdleState), "didn't transition to IdleState after start()");
     QVERIFY2((audioOutput.error() == QAudio::NoError), "error state is not equal to QAudio::NoError after start()");
     stateSignal.clear();
@@ -665,35 +750,31 @@ void tst_QAudioSink::pushSuspendResume()
     QVERIFY2((audioOutput.elapsedUSecs() > 0), "elapsedUSecs() is still zero after start()");
     QVERIFY2((audioOutput.processedUSecs() == qint64(0)), "processedUSecs() is not zero after start()");
 
+    auto firstHalfChecker = [&]() {
+        QVERIFY2((stateSignal.size() == 1),
+                 QString("didn't emit signal after receiving data, got %1 signals instead")
+                         .arg(dumpStateSignalSpy(stateSignal))
+                         .toUtf8()
+                         .constData());
+        QVERIFY2((audioOutput.state() == QAudio::ActiveState),
+                 "didn't transition to ActiveState after receiving data");
+        QVERIFY2((audioOutput.error() == QAudio::NoError),
+                 "error state is not equal to QAudio::NoError after receiving data");
+    };
+
     qint64 written = 0;
-    bool firstBuffer = true;
-
     // Play half of the clip
-    while (written < (audioFile->size() - QWaveDecoder::headerLength()) / 2) {
+    pushDataToAudioSink(audioOutput, *audioFile, *feed, written, wavDataSize(*audioFile) / 2,
+                        firstHalfChecker, true);
 
-        if (audioOutput.bytesFree() > 0) {
-            auto buffer = audioFile->read(audioOutput.bytesFree());
-            written += feed->write(buffer);
-
-            if (firstBuffer) {
-                // Check for transition to ActiveState when data is provided
-                QVERIFY2((stateSignal.count() == 1),
-                         QString("didn't emit signal after receiving data, got %1 signals instead")
-                         .arg(stateSignal.count()).toUtf8().constData());
-                QVERIFY2((audioOutput.state() == QAudio::ActiveState), "didn't transition to ActiveState after receiving data");
-                QVERIFY2((audioOutput.error() == QAudio::NoError), "error state is not equal to QAudio::NoError after receiving data");
-                firstBuffer = false;
-            }
-        } else
-            QTest::qWait(20);
-    }
     stateSignal.clear();
 
+    const auto suspendedInState = audioOutput.state();
     audioOutput.suspend();
 
-    QTRY_VERIFY2((stateSignal.count() == 1),
+    QTRY_VERIFY2((stateSignal.size() == 1),
              QString("didn't emit SuspendedState signal after suspend(), got %1 signals instead")
-             .arg(stateSignal.count()).toUtf8().constData());
+             .arg(dumpStateSignalSpy(stateSignal)).toUtf8().constData());
     QVERIFY2((audioOutput.state() == QAudio::SuspendedState), "didn't transition to SuspendedState after suspend()");
     QVERIFY2((audioOutput.error() == QAudio::NoError), "error state is not equal to QAudio::NoError after suspend()");
     stateSignal.clear();
@@ -712,27 +793,30 @@ void tst_QAudioSink::pushSuspendResume()
     QTest::qWait(20);
 
     // Check that QAudioSink immediately transitions to IdleState
-    QVERIFY2((stateSignal.count() == 1),
-             QString("didn't emit signal after resume(), got %1 signals instead").arg(stateSignal.count()).toUtf8().constData());
-    QVERIFY2((audioOutput.state() == QAudio::IdleState), "didn't transition to IdleState after resume()");
+    QVERIFY2((stateSignal.size() == 1),
+             QString("didn't emit signal after resume(), got %1 signals instead")
+                     .arg(dumpStateSignalSpy(stateSignal))
+                     .toUtf8()
+                     .constData());
+    QVERIFY2((audioOutput.state() == suspendedInState), "resume() didn't transition to state before suspend()");
     QVERIFY2((audioOutput.error() == QAudio::NoError), "error state is not equal to QAudio::NoError after resume()");
     stateSignal.clear();
 
     // Play rest of the clip
-    while (!audioFile->atEnd()) {
-        if (audioOutput.bytesFree() > 0) {
-            auto buffer = audioFile->read(audioOutput.bytesFree());
-            written += feed->write(buffer);
-            QVERIFY2((audioOutput.state() == QAudio::ActiveState), "didn't transition to ActiveState after writing audio data");
-        } else
-            QTest::qWait(20);
-    }
+
+    auto restChecker = [&]() {
+        QVERIFY2((audioOutput.state() == QAudio::ActiveState),
+                 "didn't transition to ActiveState after writing audio data");
+    };
+
+    pushDataToAudioSink(audioOutput, *audioFile, *feed, written, -1, restChecker);
+
     QVERIFY(audioOutput.state() != QAudio::IdleState);
     stateSignal.clear();
 
     QVERIFY2(audioFile->atEnd(), "didn't play to EOF");
-    QTRY_VERIFY(stateSignal.count() > 0);
-    QCOMPARE(qvariant_cast<QAudio::State>(stateSignal.last().at(0)), QAudio::IdleState);
+    QTRY_VERIFY(stateSignal.size() > 0);
+    QTRY_COMPARE(qvariant_cast<QAudio::State>(stateSignal.last().at(0)), QAudio::IdleState);
     QVERIFY2((audioOutput.state() == QAudio::IdleState), "didn't transitions to IdleState when at EOF");
     stateSignal.clear();
 
@@ -740,8 +824,11 @@ void tst_QAudioSink::pushSuspendResume()
 
     audioOutput.stop();
     QTest::qWait(40);
-    QVERIFY2((stateSignal.count() == 1),
-             QString("didn't emit StoppedState signal after stop(), got %1 signals instead").arg(stateSignal.count()).toUtf8().constData());
+    QVERIFY2((stateSignal.size() == 1),
+             QString("didn't emit StoppedState signal after stop(), got %1 signals instead")
+                     .arg(dumpStateSignalSpy(stateSignal))
+                     .toUtf8()
+                     .constData());
     QVERIFY2((audioOutput.state() == QAudio::StoppedState), "didn't transitions to StoppedState after stop()");
 
     QVERIFY2((audioOutput.error() == QAudio::NoError), "error() is not QAudio::NoError after stop()");
@@ -750,6 +837,46 @@ void tst_QAudioSink::pushSuspendResume()
     audioFile->close();
 }
 
+void tst_QAudioSink::pushResetResume()
+{
+    auto audioFile = audioFiles.at(0);
+    auto audioFormat = testFormats.at(0);
+
+    QAudioSink audioOutput(audioFormat, this);
+
+    audioOutput.setBufferSize(8192);
+    audioOutput.setVolume(0.1f);
+
+    audioFile->close();
+    audioFile->open(QIODevice::ReadOnly);
+    audioFile->seek(QWaveDecoder::headerLength());
+
+    QPointer<QIODevice> feed = audioOutput.start();
+
+    QTest::qWait(20);
+
+    auto buffer = audioFile->read(audioOutput.bytesFree());
+    feed->write(buffer);
+
+    QTest::qWait(20);
+    QTRY_COMPARE(audioOutput.state(), QAudio::ActiveState);
+
+    audioOutput.reset();
+    QCOMPARE(audioOutput.state(), QAudio::StoppedState);
+    QCOMPARE(audioOutput.error(), QAudio::NoError);
+
+    const auto processedUSecs = audioOutput.processedUSecs();
+
+    audioOutput.resume();
+    QTest::qWait(40);
+
+    // Nothing changed if resume after reset
+    QCOMPARE(audioOutput.state(), QAudio::StoppedState);
+    QCOMPARE(audioOutput.error(), QAudio::NoError);
+
+    QCOMPARE(audioOutput.processedUSecs(), processedUSecs);
+}
+
 void tst_QAudioSink::pushUnderrun()
 {
     QFETCH(FilePtr, audioFile);
@@ -773,8 +900,11 @@ void tst_QAudioSink::pushUnderrun()
     QIODevice* feed = audioOutput.start();
 
     // Check that QAudioSink immediately transitions to IdleState
-    QTRY_VERIFY2((stateSignal.count() == 1),
-                 QString("didn't emit signal on start(), got %1 signals instead").arg(stateSignal.count()).toUtf8().constData());
+    QTRY_VERIFY2((stateSignal.size() == 1),
+                 QString("didn't emit signal on start(), got %1 signals instead")
+                         .arg(dumpStateSignalSpy(stateSignal))
+                         .toUtf8()
+                         .constData());
     QVERIFY2((audioOutput.state() == QAudio::IdleState), "didn't transition to IdleState after start()");
     QVERIFY2((audioOutput.error() == QAudio::NoError), "error state is not equal to QAudio::NoError after start()");
     stateSignal.clear();
@@ -785,64 +915,59 @@ void tst_QAudioSink::pushUnderrun()
     QVERIFY2((audioOutput.processedUSecs() == qint64(0)), "processedUSecs() is not zero after start()");
 
     qint64 written = 0;
-    bool firstBuffer = true;
-    QByteArray buffer(AUDIO_BUFFER, 0);
 
     // Play half of the clip
-    while (written < (audioFile->size() - QWaveDecoder::headerLength()) / 2) {
 
-        if (audioOutput.bytesFree() > 0) {
-            auto buffer = audioFile->read(audioOutput.bytesFree());
-            written += feed->write(buffer);
+    auto firstHalfChecker = [&]() {
+        QVERIFY2((stateSignal.size() == 1),
+                 QString("didn't emit signal after receiving data, got %1 signals instead")
+                         .arg(dumpStateSignalSpy(stateSignal))
+                         .toUtf8()
+                         .constData());
+        QVERIFY2((audioOutput.state() == QAudio::ActiveState),
+                 "didn't transition to ActiveState after receiving data");
+        QVERIFY2((audioOutput.error() == QAudio::NoError),
+                 "error state is not equal to QAudio::NoError after receiving data");
+    };
+
+    pushDataToAudioSink(audioOutput, *audioFile, *feed, written, wavDataSize(*audioFile) / 2,
+                        firstHalfChecker, true);
 
-            if (firstBuffer) {
-                // Check for transition to ActiveState when data is provided
-                QVERIFY2((stateSignal.count() == 1),
-                         QString("didn't emit signal after receiving data, got %1 signals instead")
-                         .arg(stateSignal.count()).toUtf8().constData());
-                QVERIFY2((audioOutput.state() == QAudio::ActiveState), "didn't transition to ActiveState after receiving data");
-                QVERIFY2((audioOutput.error() == QAudio::NoError), "error state is not equal to QAudio::NoError after receiving data");
-                firstBuffer = false;
-            }
-        } else
-            QTest::qWait(20);
-    }
     stateSignal.clear();
 
     // Wait for data to be played
     QTest::qWait(700);
 
-    QVERIFY2((stateSignal.count() == 1),
+    QVERIFY2((stateSignal.size() == 1),
              QString("didn't emit IdleState signal after suspend(), got %1 signals instead")
-             .arg(stateSignal.count()).toUtf8().constData());
+             .arg(dumpStateSignalSpy(stateSignal)).toUtf8().constData());
     QVERIFY2((audioOutput.state() == QAudio::IdleState), "didn't transition to IdleState, no data");
     QVERIFY2((audioOutput.error() == QAudio::UnderrunError), "error state is not equal to QAudio::UnderrunError, no data");
     stateSignal.clear();
 
-    firstBuffer = true;
     // Play rest of the clip
-    while (!audioFile->atEnd()) {
-        if (audioOutput.bytesFree() > 0) {
-            auto buffer = audioFile->read(audioOutput.bytesFree());
-            written += feed->write(buffer);
-            if (firstBuffer) {
-                // Check for transition to ActiveState when data is provided
-                QVERIFY2((stateSignal.count() == 1),
-                         QString("didn't emit signal after receiving data, got %1 signals instead")
-                         .arg(stateSignal.count()).toUtf8().constData());
-                QVERIFY2((audioOutput.state() == QAudio::ActiveState), "didn't transition to ActiveState after receiving data");
-                QVERIFY2((audioOutput.error() == QAudio::NoError), "error state is not equal to QAudio::NoError after receiving data");
-                firstBuffer = false;
-            }
-        } else
-            QTest::qWait(20);
-    }
+    auto restChecker = [&]() {
+        QVERIFY2((stateSignal.size() == 1),
+                 QString("didn't emit signal after receiving data, got %1 signals instead")
+                         .arg(dumpStateSignalSpy(stateSignal))
+                         .toUtf8()
+                         .constData());
+        QVERIFY2((audioOutput.state() == QAudio::ActiveState),
+                 "didn't transition to ActiveState after receiving data");
+        QVERIFY2((audioOutput.error() == QAudio::NoError),
+                 "error state is not equal to QAudio::NoError after receiving data");
+    };
+    pushDataToAudioSink(audioOutput, *audioFile, *feed, written, -1, restChecker, true);
+
     stateSignal.clear();
 
     // Wait until playback finishes
     QVERIFY2(audioFile->atEnd(), "didn't play to EOF");
-    QTRY_VERIFY2((stateSignal.count() == 1),
-             QString("didn't emit IdleState signal when at EOF, got %1 signals instead").arg(stateSignal.count()).toUtf8().constData());
+    QTRY_VERIFY2((stateSignal.size() == 1),
+                 QString("didn't emit IdleState signal when at EOF, got %1 signals instead")
+                         .arg(dumpStateSignalSpy(stateSignal))
+                         .toUtf8()
+                         .constData());
     QVERIFY2((audioOutput.state() == QAudio::IdleState), "didn't transitions to IdleState when at EOF");
     stateSignal.clear();
 
@@ -850,8 +975,11 @@ void tst_QAudioSink::pushUnderrun()
 
     audioOutput.stop();
     QTest::qWait(40);
-    QVERIFY2((stateSignal.count() == 1),
-             QString("didn't emit StoppedState signal after stop(), got %1 signals instead").arg(stateSignal.count()).toUtf8().constData());
+    QVERIFY2((stateSignal.size() == 1),
+             QString("didn't emit StoppedState signal after stop(), got %1 signals instead")
+                     .arg(dumpStateSignalSpy(stateSignal))
+                     .toUtf8()
+                     .constData());
     QVERIFY2((audioOutput.state() == QAudio::StoppedState), "didn't transitions to StoppedState after stop()");
 
     QVERIFY2((audioOutput.error() == QAudio::NoError), "error() is not QAudio::NoError after stop()");
